/**
@author   Cody Johnson
@email    cody.ele@noritake.com (Feel free to ask questions)
@website  noritake-elec.com
@version  v1.0
@ide      IAR
@brief    SPI library for STM32F4xx and Noritake GU-D Series Modules

@vertatim
Provide the SPI interface between STM32F4Discovery and Noritake GU-D modules.
This library uses the STM32F4Discovery library functions from stm32f4xx_spi.
@endverbatim
*/

#include "GUD_SPI.h"
#include "stdint.h"
#include <stddef.h>

void Delay(__IO uint32_t time);
extern __IO uint32_t timingDelay;

void Delay(__IO uint32_t time){
  timingDelay = time;
  while(timingDelay != 0);
}

/**
Initialize SPI communication.

@return none
*/
void GUD_SPI_Init(){
  interface = 1;
  SysTick_Config(SystemCoreClock / 400000);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
  
  SPI_InitTypeDef SPI_InitTypeDefStruct;
  
  // Set up SPI parameters for SPI1.
  SPI_InitTypeDefStruct.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_256;
  SPI_InitTypeDefStruct.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitTypeDefStruct.SPI_Mode = SPI_Mode_Master;
  SPI_InitTypeDefStruct.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitTypeDefStruct.SPI_NSS = SPI_NSS_Soft;
  SPI_InitTypeDefStruct.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitTypeDefStruct.SPI_CPOL = SPI_CPOL_High;
  SPI_InitTypeDefStruct.SPI_CPHA = SPI_CPHA_2Edge;
  SPI_Init(SPI1, &SPI_InitTypeDefStruct);
  
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB | RCC_AHB1Periph_GPIOE, ENABLE);
  
  GPIO_InitTypeDef GPIO_InitTypeDefStruct;
  GPIO_InitTypeDef GPIO_InitTypeDefStruct2;
  GPIO_InitTypeDef GPIO_InitTypeDefStruct3;

  // SCK, MISO, MOSI configuration
  // SCK = PA5
  // MISO = PA6
  // MOSI = PA7
  GPIO_InitTypeDefStruct.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
  GPIO_InitTypeDefStruct.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitTypeDefStruct.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitTypeDefStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitTypeDefStruct.GPIO_OType = GPIO_OType_PP;
  GPIO_Init(GPIOA, &GPIO_InitTypeDefStruct);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource5, GPIO_AF_SPI1);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_SPI1);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_SPI1);
  
  // Chipselect configuration
  // CS = PE3
  GPIO_InitTypeDefStruct2.GPIO_Pin = GPIO_Pin_3;
  GPIO_InitTypeDefStruct2.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitTypeDefStruct2.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitTypeDefStruct2.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitTypeDefStruct2.GPIO_OType = GPIO_OType_PP;
  GPIO_Init(GPIOE, &GPIO_InitTypeDefStruct2);
  GPIO_SetBits(GPIOE, GPIO_Pin_3); // Set CS
  
  // MBUSY configuration
  // MBUSY = PB3
  GPIO_InitTypeDefStruct3.GPIO_Pin = GPIO_Pin_3;
  GPIO_InitTypeDefStruct3.GPIO_Mode = GPIO_Mode_IN;
  GPIO_Init(GPIOB, &GPIO_InitTypeDefStruct3);
  
  // Configure SCK
  // This function is separated from MOSI and MISO due to timing constraints
  
  
  SPI_Cmd(SPI1, ENABLE);
}

/**
Start SPI communication by lowering CS and sending 0x44.

@return none
*/
void startSPI(){
  GPIO_ResetBits(GPIOE, GPIO_Pin_3); //Clear CS
  writeSPI(0x44);
}

/**
End SPI communication by raising CS.

@return none
*/
void endSPI(){
  GPIO_SetBits(GPIOE, GPIO_Pin_3); //Set CS
}

/**
Initialize the GUD unit.

@return none
*/
void initGUD(){
  writeSPI(0x1b);
  writeSPI(0x40);
}

/**
Write a byte to the display.

@param  data The data to be written.
@return none
*/
void writeSPI(uint8_t data){
  while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_3) == Bit_SET); // wait for MBUSY to go low
  while(!SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE));
  SPI_I2S_SendData(SPI1, data);
  while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_3) == Bit_RESET); // wait for MBUSY to go high
  while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_3) == Bit_SET); // wait for MBUSY to go low
  //Delay(1);
}

/**
Write a byte to the display.
This function does not chech to see if MBUSY goes high.

@param data The data to be written.
@return none
*/
void writeSPI2(uint8_t data){
  while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_3) == Bit_SET); // wait for MBUSY to go low
  while(!SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE));
  SPI_I2S_SendData(SPI1, data);
  while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_3) == Bit_SET); // wait for MBUSY to go low
  Delay(2);
}

/**
Read a byte from the display.

@return The data byte that was read.
*/
uint8_t readSPI(){
  uint8_t data;
  while(!SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE));
  SPI_I2S_SendData(SPI1, 0x00); // Send dummy data to generate clock
  while(!SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE));
  while(!SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE));
  data = SPI_I2S_ReceiveData(SPI1);
  while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_3) == Bit_RESET); // wait for MBUSY to go high
  while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_3) == Bit_SET); // wait for MBUSY to go low
  //Delay(2);
  return data;
}

/**
Write an array of bytes to the display.
@param  data The data to be written.
@return none
*/
void writeString(char* data){
  int i = 0;
  while(data[i] != NULL){
    writeSPI(data[i]);
    i++;
  }
}

/**
Read the status byte from the display.
This is required to read data.
The status data indicates the number of bytes to be read.

@return none
*/
uint8_t readSPIstatus(){
  uint8_t incomingByte = 0;
  uint8_t bytesToRead = 0;
  GPIO_ResetBits(GPIOE, GPIO_Pin_3); //Clear CS
  writeSPI(0x58);
  incomingByte = readSPI();
  endSPI();
  if((incomingByte & 0xc0) == 0){
    bytesToRead = incomingByte & 0x3f;
  }
  return bytesToRead;
}

/**
Start the read sequence by lowering CS and transmitting 0x54.

@return none
*/
void startSPIRead(){
  GPIO_ResetBits(GPIOE, GPIO_Pin_3); //Clear CS
  writeSPI(0x54);
}

/**
Reads a byte of data from the display.
This function should be used after startSPIRead() has been called.

@return The data byte that was read.
*/
uint8_t readSPIData(){
  uint8_t rxdata;
  rxdata = readSPI();
  return rxdata;
}